==> framework-plugin-low-code-0.7.2-beta.0.tgz: package/template/src/handlers/utils/common.js <==
import config from '../../datasources/config'
import { getTcbInstance, getAccessToken } from '@cloudbase/weda-cloud-sdk/dist/h5';

async function loginScope() {
  const { auth } = await getTcbInstance();
  return auth.loginScope();
}

export function getComponentId(key) {
  return `__weapps-component-wrapper-${key}`;
}

export const pathSpecialSymbol = '__$__';

export function pathTransformDotToSymbol(str) {
  return str.replace(/\./g, pathSpecialSymbol);
}

export function pathTransformSymbolToDot(str) {
  return String(str).replace(new RegExp(`__\\$__`, 'g'), '.');
}

/**
 * All data bindings are generated as functions: (forItems, event?) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(
  dataBinds,
  forItems,
  codeContext,
  scopeContext,
  throwError,
  context
) {
  const resolvedProps = {};
  for (const prop in dataBinds) {
    let fn = dataBinds[prop];
    try {
      if (codeContext && codeContext.$WEAPPS_COMP) {
        fn = fn.bind(codeContext.$WEAPPS_COMP);
      }
      resolvedProps[prop] = fn(
        forItems,
        codeContext && codeContext.event,
        context
      );
    } catch (e) {
      console.error('Error resolving data binding', prop, dataBinds[prop], e);
      if (throwError) {
        throw e;
      }
    }
  }
  return resolvedProps;
}

const varSeparator = '.';
export function getDeep(target, key) {
  if (key == null) {
    return target;
  }
  const keys = (key + '').split(varSeparator);
  let prop = target[keys[0]];
  for (let i = 1; i < keys.length; i++) {
    prop = prop[keys[i]];
  }
  return prop;
}

/**
 * 用于处理自定义组件props传参结构，对系统变量进行保留
 */
export function resolveComponentProps(props, isPlainProps) {
  const { staticResourceAttribute } = props;
  staticResourceAttribute &&
    staticResourceAttribute.map((property) => {
      if (props.data && props.data[property]) {
        props.data[property] = getStaticResourceAttribute(props.data[property]);
      }
    });
  if (!isPlainProps) {
    return {
      ...props,
    };
  }
  const { data = {}, events = [], $node, ...restProps } = props;
  const customProps = { ...data };
  const builtinProps = [
    // react 保留字
    'ref',
    'key',
    'dangerouslySetInnerHTML',
    'className',
    'htmlFor',
    'style',
    'contentEditable',
    // lowcode 保留字
    'events',
    'children',
    '_parentId',
    '_visible',
    'classList',
    'widgetType',
    'getWidgetsByType',
    'getDom',
    'domRef',
    'extends',
    // 小程序保留字
    'id',
    'class',
    'hidden',
    'slot',
  ];
  builtinProps.map((prop) => delete customProps[prop]);
  // 选区events处理
  const { _selectableBlockEvents, ...restData } = data;
  const { emit: _selectableBlockEmit } = _selectableBlockEvents;
  _selectableBlockEvents.events = new Proxy(
    _selectableBlockEvents.events.reduce((events, item) => {
      const propName = item;
      events[propName] = (e) => _selectableBlockEmit(propName, e);
      return events;
    }, {}),
    {
      get(obj, prop) {
        return prop in obj ? obj[prop] : (e) => _selectableBlockEmit(prop, e);
      },
    }
  );
  return {
    ...restData,
    ...restProps,
    _selectableBlockEvents,
    events: events.reduce((events, item) => {
      const propName = item;
      events[propName] = (e) => restProps.emit(propName, e);
      return events;
    }, {}),
  };
}

export function getStaticResourceAttribute(staticUrl) {
  if (/^\//.test(staticUrl)) {
    const { domain = '' } = window.app || {};
    const url = `https://${domain}${staticUrl}`;
    return url;
  }
  return staticUrl;
}

export function findLoginPage(app = window.app) {
  const { pages = [] } = app.utils._getConfig();
  return pages.find(item => item.type === 'login');
}

let _AUTH_CONFIG_CACHE = null;
export async function getAuthConfig(app = window.app) {
  if (_AUTH_CONFIG_CACHE) {
    return _AUTH_CONFIG_CACHE;
  }
  try {
    const res = await app.cloud.callWedaApi({
      action: "DescribeRuntimeResourceStrategy",
      data: {
        ResourceType: `<%= isAdminPortal? 'modelApp' : 'app'%>`,
        ResourceId: app.id,
      },
    });
    const settingData = {};
    // 云api不支持map只能传字符串，需要转换
    res.forEach((item) => {
      settingData[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
    });
    _AUTH_CONFIG_CACHE = settingData;
    return _AUTH_CONFIG_CACHE;
  } catch (e) {
    return {
      NeedLogin: false,
      RejectStrategy: "show_warning",
    };
  }
}

let _AUTH_CACHE_MAP = {}
async function getAccessPermission(app, appId, pageId) {
  const cacheKey = `${appId}-${pageId}`
  if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
    return _AUTH_CACHE_MAP[cacheKey];
  }

  let isAccess = false;
  try {
    const res = await app.cloud.callWedaApi({
      action: 'DescribeResourcesPermission',
      data: {
        ResourceType: `<%= isAdminPortal? 'modelApp' : 'app'%>`,
        ResourceIdList: [cacheKey],
        AppResourceId: appId,
      },
    });
    if (Array.isArray(res) && res.length > 0) {
      isAccess = !!res[0].IsAccess;
    }
    _AUTH_CACHE_MAP[cacheKey] = isAccess;
  } catch (e) {
    console.warn('getAccessPermission', e);
  }
  return isAccess
}

/**
 * 检查页面权限
 **/
export async function checkAuth(app, appId, $page) {
  <% if (isAdminPortal || isXPage) { %>return true;<% } %>
  const loginPage = findLoginPage(app);
  if (loginPage?.id === $page.id) {
    return true
  }
  app.showNavigationBarLoading();
  const requestList = [getAccessPermission(app, appId, $page.id)];
  // 暂时先认为有登录页则自定义登录功能开启且生效
  if (loginPage) {
    requestList.push(getAuthConfig(app));
  }
  const [isAccess, authConfig] = await Promise.all(requestList);
  app.hideNavigationBarLoading();

  if (!isAccess) {
    if (loginPage && (authConfig.NeedLogin || authConfig.RejectStrategy == 'to_login')) {
      redirectToLogin($page);
    } else {
      app.showToast({
        title: '页面无访问权限',
        icon: 'error',
      });
    }
  } else if (loginPage && authConfig.NeedLogin) {
    // 此分支逻辑本不应该前端判断是否登录，历史原因后端短期内搞不定，后续后端优化后删除
    try {
      const [scope, { accessToken }] = await Promise.all([loginScope(), getAccessToken()]);
      if (!accessToken || scope === 'anonymous') {
        redirectToLogin($page);
      }
    } catch (e) {
      console.error('获取身份失败', e);
      redirectToLogin($page);
    }
  }
  return isAccess;
}

export function redirectToLogin(currentPage) {
  // 去登录则清空权限缓存。
  _AUTH_CACHE_MAP = {};
  const app = window.app;
  const loginPage = findLoginPage(app);
  if (!currentPage) {
    currentPage = app.utils.getCurrentPage() || {};
  }
  if (loginPage?.id === currentPage.id) {
    return true
  }
  if (loginPage) {
    app.redirectTo({
      pageId: loginPage.id,
      params: {
        sourcePageId: currentPage.id,
        sourcePageParams: currentPage.dataset.params
      }
    })
  } else {
    app.showToast({
      title: '用户未登录',
      icon: 'error',
    });
  }
}

const _REPORTED = {}
export function reportTime(tag, time, only = false) {
  if (!window._aegis || !tag) {
    return;
  }
  if (window['_WedaHostConfig'] && !window['_WedaHostConfig']['_REPORTED']) {
    window['_WedaHostConfig']['_REPORTED'] = _REPORTED
  }

  const CACHE_MAP = window['_WedaHostConfig']?.['_REPORTED'] || _REPORTED

  if (only && CACHE_MAP[tag]) {
    return;
  }
  CACHE_MAP[tag] = true
  try {
    let t = time === undefined ? (performance?.now?.() || Date.now() - window['_aegis_inited']) : time;
    window._aegis.reportTime({
      name: tag,
      duration: t,
      ext2: config.isProd ? 'production' : 'preview',
    })
  } catch (e) {
    console.log(e)
  }
}

==> framework-plugin-low-code-0.7.8-beta.0.tgz: package/template/src/handlers/utils/common.js <==
import config from '../../datasources/config'
import { getTcbInstance, getAccessToken } from '@cloudbase/weda-cloud-sdk/dist/h5';

async function loginScope() {
  const { auth } = await getTcbInstance();
  return auth.loginScope();
}

export function getComponentId(key) {
  return `__weapps-component-wrapper-${key}`;
}

export const pathSpecialSymbol = '__$__';

export function pathTransformDotToSymbol(str) {
  return str.replace(/\./g, pathSpecialSymbol);
}

export function pathTransformSymbolToDot(str) {
  return String(str).replace(new RegExp(`__\\$__`, 'g'), '.');
}

/**
 * All data bindings are generated as functions: (forItems, event?) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(
  dataBinds,
  forItems,
  codeContext,
  scopeContext,
  throwError,
  context
) {
  const resolvedProps = {};
  for (const prop in dataBinds) {
    let fn = dataBinds[prop];
    try {
      if (codeContext && codeContext.$WEAPPS_COMP) {
        fn = fn.bind(codeContext.$WEAPPS_COMP);
      }
      resolvedProps[prop] = fn(
        forItems,
        codeContext && codeContext.event,
        context
      );
    } catch (e) {
      console.error('Error resolving data binding', prop, dataBinds[prop], e);
      if (throwError) {
        throw e;
      }
    }
  }
  return resolvedProps;
}

const varSeparator = '.';
export function getDeep(target, key) {
  if (key == null) {
    return target;
  }
  const keys = (key + '').split(varSeparator);
  let prop = target[keys[0]];
  for (let i = 1; i < keys.length; i++) {
    prop = prop[keys[i]];
  }
  return prop;
}

/**
 * 用于处理自定义组件props传参结构，对系统变量进行保留
 */
export function resolveComponentProps(props, isPlainProps) {
  const { staticResourceAttribute } = props;
  staticResourceAttribute &&
    staticResourceAttribute.map((property) => {
      if (props.data && props.data[property]) {
        props.data[property] = getStaticResourceAttribute(props.data[property]);
      }
    });
  if (!isPlainProps) {
    return {
      ...props,
    };
  }
  const { data = {}, events = [], $node, ...restProps } = props;
  const customProps = { ...data };
  const builtinProps = [
    // react 保留字
    'ref',
    'key',
    'dangerouslySetInnerHTML',
    'className',
    'htmlFor',
    'style',
    'contentEditable',
    // lowcode 保留字
    'events',
    'children',
    '_parentId',
    '_visible',
    'classList',
    'widgetType',
    'getWidgetsByType',
    'getDom',
    'domRef',
    'extends',
    // 小程序保留字
    'id',
    'class',
    'hidden',
    'slot',
  ];
  builtinProps.map((prop) => delete customProps[prop]);
  // 选区events处理
  const { _selectableBlockEvents, ...restData } = data;
  const { emit: _selectableBlockEmit } = _selectableBlockEvents;
  _selectableBlockEvents.events = new Proxy(
    _selectableBlockEvents.events.reduce((events, item) => {
      const propName = item;
      events[propName] = (e) => _selectableBlockEmit(propName, e);
      return events;
    }, {}),
    {
      get(obj, prop) {
        return prop in obj ? obj[prop] : (e) => _selectableBlockEmit(prop, e);
      },
    }
  );
  return {
    ...restData,
    ...restProps,
    _selectableBlockEvents,
    events: events.reduce((events, item) => {
      const propName = item;
      events[propName] = (e) => restProps.emit(propName, e);
      return events;
    }, {}),
  };
}

export function getStaticResourceAttribute(staticUrl) {
  const { __internal__ = {} } = window.app || {};
  return __internal__?.resolveStaticResourceUrl?.(staticUrl) || staticUrl
}

export function findLoginPage(app = window.app) {
  const { pages = [] } = app.__internal__.getConfig();
  return pages.find(item => item.type === 'login');
}

let _AUTH_CONFIG_CACHE = null;
export async function getAuthConfig(app = window.app) {
  if (_AUTH_CONFIG_CACHE) {
    return _AUTH_CONFIG_CACHE;
  }
  try {
    const res = await app.cloud.callWedaApi({
      action: "DescribeRuntimeResourceStrategy",
      data: {
        ResourceType: `<%= isAdminPortal? 'modelApp' : 'app'%>`,
        ResourceId: app.id,
      },
    });
    const settingData = {};
    // 云api不支持map只能传字符串，需要转换
    res.forEach((item) => {
      settingData[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
    });
    _AUTH_CONFIG_CACHE = settingData;
    return _AUTH_CONFIG_CACHE;
  } catch (e) {
    return {
      NeedLogin: false,
      RejectStrategy: "show_warning",
    };
  }
}

let _AUTH_CACHE_MAP = {}
async function getAccessPermission(app, appId, pageId) {
  const cacheKey = `${appId}-${pageId}`
  if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
    return _AUTH_CACHE_MAP[cacheKey];
  }

  let isAccess = false;
  try {
    const res = await app.cloud.callWedaApi({
      action: 'DescribeResourcesPermission',
      data: {
        ResourceType: `<%= isAdminPortal? 'modelApp' : 'app'%>`,
        ResourceIdList: [cacheKey],
        AppResourceId: appId,
      },
    });
    if (Array.isArray(res) && res.length > 0) {
      isAccess = !!res[0].IsAccess;
    }
    _AUTH_CACHE_MAP[cacheKey] = isAccess;
  } catch (e) {
    console.warn('getAccessPermission', e);
  }
  return isAccess
}

export async function checkAnonymous() {
  // 用户是否非匿名登录前端判断条件：有accessToken且scope不为anonymous
  // 此分支逻辑本不应该前端判断是否登录，历史原因后端短期内搞不定，后续后端优化后删除
  let isAnonymous = true;
  try {
    const [scope, { accessToken }] = await Promise.all([loginScope(), getAccessToken()]);
    if (accessToken && scope !== 'anonymous') {
      isAnonymous = false;
    }
  } catch (e) { }
  return isAnonymous;
}

let loginConfig;
export async function getLoginConfig() {
  if (loginConfig !== undefined) {
    return loginConfig;
  }
  const { staticResourceDomain, id } = app.__internal__.getConfig();
  const url = `https://${staticResourceDomain}/${id}/login.config.json?t=${Date.now()}`;
  try {
    let response = await fetch(url);
    loginConfig = await response.json();
  } catch (e) {
    loginConfig = {};
    console.error(`加载登录配置${url}失败`, e);
  }
  return loginConfig;
}

/**
 * 检查页面权限
 **/
export async function checkAuth(app, appId, $page) {
  <% if (isAdminPortal || isXPage) { %>return true;<% } %>
    app.showNavigationBarLoading();
  const loginConfig = await getLoginConfig();
  const requestList = [getAccessPermission(app, appId, $page.id)];
  // 登录配置内有web登录项则认为开启web登录
  if (loginConfig.web?.length > 0) {
    requestList.push(getAuthConfig(app));
    requestList.push(checkAnonymous());
    requestList.push(getLoginConfig()); // 提前并行加载
  }
  const [isAccess, authConfig, isAnonymous] = await Promise.all(requestList);
  app.hideNavigationBarLoading();

  if (loginConfig.web?.length > 0 && isAnonymous) {
    if (authConfig.NeedLogin || (!isAccess && authConfig.RejectStrategy === 'to_login')) {
      redirectToLogin($page);
      return false;
    }
  }

  if (!isAccess) {
    app.showToast({
      title: '页面无访问权限',
      icon: 'error',
    });
  }
  return isAccess;
}

export async function redirectToLogin(currentPage) {
  // 去登录则清空权限缓存。
  _AUTH_CACHE_MAP = {};
  const app = window.app;

  const loginConfig = await getLoginConfig();
  if (loginConfig.web?.length > 0) {
    const { staticResourceDomain, envId, id, clientId } = app.__internal__.getConfig();
    const url = `${location.origin}/__auth/?redirect_uri=${encodeURIComponent(
      location.href,
    )}&app_id=${id}&env_id=${envId}&client_id=${clientId}&s_domain=${staticResourceDomain}`;
    app.redirectTo({
      mode: 'web',
      params: {},
      url,
    });
  } else {
    app.showToast({
      title: '用户未登录',
      icon: 'error',
    });
  }
}

const _REPORTED = {}
export function reportTime(tag, time, only = false) {
  if (!window._aegis || !tag) {
    return;
  }
  if (window['_WedaHostConfig'] && !window['_WedaHostConfig']['_REPORTED']) {
    window['_WedaHostConfig']['_REPORTED'] = _REPORTED
  }

  const CACHE_MAP = window['_WedaHostConfig']?.['_REPORTED'] || _REPORTED

  if (only && CACHE_MAP[tag]) {
    return;
  }
  CACHE_MAP[tag] = true
  try {
    let t = time === undefined ? (performance?.now?.() || Date.now() - window['_aegis_inited']) : time;
    window._aegis.reportTime({
      name: tag,
      duration: t,
      ext2: config.isProd ? 'production' : 'preview',
    })
  } catch (e) {
    console.log(e)
  }
}
